/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # flows.c - Flow Management.                       #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <stdio.h>

#include "flows.h"
#include "common.h"

p_flist_head new_flist(p_flow item) {
	p_flist_head list;
	p_flist first;
	p_flow new_item;

	list = (p_flist_head)malloc(sizeof(flist_head));
	list->elements = 0;
	list->max_seq = 0;
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Trying to create an empty list\n");
#endif
		list->head = 0;
		list->tail = 0;
		return list;
	}
	first = (p_flist)malloc(sizeof(flist));
	new_item = (p_flow)malloc(sizeof(flow));
	if (!new_item || !list || !first) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return NULL;
	}
	list->head = first;
	list->tail = first;
	list->actual = first;
	memcpy(new_item, item, sizeof(flow));
	first->item = new_item;
	first->next = 0;
	first->prev = 0;
	list->elements++;
	return list;
}

p_flist_head copy_flow(p_flist_head list, int my_flow) {
	p_flist_head new_list;
	p_flow tmp;
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Trying to copy an empty list\n");
#endif
		return NULL;
	}
	new_list = (p_flist_head)malloc(sizeof(flist_head));
	new_list->elements = 0;
	new_list->max_seq = 0;
	new_list->head = 0;
	new_list->tail = 0;
	if (!new_list) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return NULL;
	}
	tmp = go_first(list);
	do {
		if (tmp->flow_id == my_flow) {
			add_item(new_list, tmp);
		}
		tmp = go_next(list);
	} while (tmp);
	new_list->actual = new_list->head;
	return new_list;
}

int del_list(p_flist_head list) {
	p_flist tmp, next;

	tmp = list->head;
	while (tmp != list->tail) {
		free (tmp->item);
		next = tmp->next;
		free(tmp);
		tmp = next;
	}
	free(tmp->item);
	free(tmp);

	free(list);
	return OK;
}

int add_item(p_flist_head list, p_flow item) {
	p_flist new_item;
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Give me a list!\n");
#endif
		return -MYEINVAL;
	}
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Inserting NULL item?, NO, i don't\n");
#endif
		return -MYEINVAL;
	}

	new_item = (p_flist)malloc(sizeof(flist));
	if (!new_item) {
#ifdef DEBUG
		fprintf(stderr,"I don't have enought memory, sorry!\n");
#endif
		return -MYEINVAL;
	}
	new_item->item = (p_flow)malloc(sizeof(flow));
	memcpy(new_item->item, item, sizeof(flow));

	new_item->next = 0;
	if (list->tail) {
		new_item->prev = list->tail;
		list->tail->next = new_item;
	} else {	// New List
		new_item->prev = 0;
		list->head = new_item;
	}
	list->tail = new_item;
	list->elements++;
	if (item->seq_num > list->max_seq)
		list->max_seq = item->seq_num;
	return OK;
}

p_flow go(p_flist_head list, p_flow item) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (!item) {
#ifdef DEBUG
		fprintf(stderr,"Item not found (is NULL\n)");
#endif
		return NULL;
	}
	list->actual = list->head;
	while (list && list->actual->item != item)
		list->actual = list->actual->next;
	if (list->actual->item != item)
		return NULL;
	return list->actual->item;
}
p_flow go_first(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	list->actual = list->head;
	if (list->actual)
		return list->actual->item;
	return NULL;
}

p_flow go_last(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	list->actual = list->tail;
	if (list->actual)
		return list->actual->item;
	return NULL;
}

p_flow go_next(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (!list->actual)
		list->actual = list->head;
	else
		list->actual = list->actual->next;
	if (!list->actual) {
		list->actual = list->tail;
		return NULL;
	}
	return list->actual->item;
}

p_flow go_prev(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (!list->actual)
		list->actual = list->tail;
	else
		list->actual = list->actual->prev;
	if (!list->actual) {
		list->actual = list->head;
		return NULL;
	}
	return list->actual->item;
}

p_flow give_flow(p_flist_head list) {
	if (!list) {
#ifdef DEBUG
		fprintf(stderr,"Can't go first item of a NULL list!\n");
#endif
		return NULL;
	}
	if (list->actual)
		return list->actual->item;
	else
		return NULL;
}

p_flow make_flow(char *data) {
	p_flow item;
	char *tmp;

	item = (p_flow)malloc(sizeof(flow));
	if (!data) {
#ifdef DEBUG
		fprintf(stderr, "NULL data is no FLOW\n");
#endif
		free(item);
		return NULL;
	}
	tmp = strstr(data, FLOW_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Flow identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(FLOW_STR);
	item->flow_id = getInteger(tmp);

	tmp = strstr(tmp, SEQ_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Sequence identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SEQ_STR);
	item->seq_num = getInteger(tmp);

	tmp = strstr(tmp, SRC_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in SOURCE identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SRC_STR);
	item->protocol = getIP(tmp, &(item->s_addr));

	tmp = strstr(tmp, DEST_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in DESTINATION identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(DEST_STR);
	getIP(tmp, &(item->d_addr));

	tmp = strstr(tmp, TX_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in TxTime identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(TX_STR);
	getTime(tmp, &(item->tx_time));

	tmp = strstr(tmp, RX_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in RxTime identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(RX_STR);
	getTime(tmp, &(item->rx_time));

	tmp = strstr(tmp, SIZE_STR);
	if (!tmp) {
#ifdef DEBUG
		fprintf(stderr, "Invalid string format in Size identification\n");
#endif
		free(item);
		return NULL;
	}
	tmp += strlen(SIZE_STR);
	item->size = getInteger(tmp);
	return item;
}

p_flist_head make_flow_list(int fd, int is_verbose) {
	int ok;
	char line[MAX_LINE];
	p_flist_head hList;
	p_flow myFlow;
	hList = new_flist(NULL);
	do {
		bzero(line,MAX_LINE);
		ok = read_line(fd, line, is_verbose);
		if (ok >= 0) {
			myFlow = make_flow(line);
			if (myFlow) {
				add_item(hList,myFlow);
			}
		}
	} while(ok > 0);
	return hList;
}
