/* ####################################################
   # Copyright 2002, 2003 Ren Serral,                #
   #                                                  #
   # flows.h - Flow Management.                       #
   #       v2.0 (04-2002)                             #
   #       v2.01(02-2003)                             #
   ####################################################

   This file is part of NetMeter.

   NetMeter is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   NetMeter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with NetMeter; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <sys/socket.h>
#include <netinet/in.h>

#ifndef __FLOWS_H__
#define __FLOWS_H__

#define FLOW_STR "Flow>"
#define SEQ_STR  "Seq>"
#define SRC_STR  "Src>"
#define DEST_STR "Dest>"
#define TX_STR   "TxTime>"
#define RX_STR   "RxTime>"
#define SIZE_STR "Size>"

typedef union u_addr {
	struct sockaddr_in s_address;
	struct sockaddr_in6 s6_address;
} address, *p_address;

typedef struct t_flow {
	long int flow_id;
	long int seq_num;
	int protocol;
	address s_addr;
	address d_addr;
	struct timeval tx_time, rx_time;
	int size;
} flow, *p_flow;

struct t_flow_list {
	p_flow item;
	struct t_flow_list *next, *prev;
};

typedef struct t_flow_list flist, *p_flist;

typedef struct t_flow_list_head {
	int elements;
	int max_seq;
	p_flist head, tail, actual;
} flist_head, *p_flist_head;

p_flist_head new_flist(p_flow item);
p_flist_head copy_flow(p_flist_head list, int flow);
int	     del_list(p_flist_head list);
int	     add_item(p_flist_head list, p_flow item);
p_flow       go(p_flist_head list, p_flow item);
p_flow       go_first(p_flist_head list);
p_flow       go_last(p_flist_head list);
p_flow       go_next(p_flist_head list);
p_flow       go_prev(p_flist_head list);
p_flow       give_flow(p_flist_head list);
p_flow       make_flow(char *data);

p_flist_head make_flow_list(int fd, int is_verbose);

#endif
