###############################################
# Copyright 2002, 2003 Ren Serral,           #
#                      Roberto Borgione       #
#                                             #
# delayu6a.tcl - Unicast   Delay IPv6 Manager #
#       v2.0 (04-2002)                        #
#       v2.01(10-2002)                        #
###############################################

#   This file is part of NetMeter.
#
#   NetMeter is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   NetMeter is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with NetMeter; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

###################################################################          
#------------------------------------------------------------------
#        RUNDELAYTEST for Ipv6
#------------------------------------------------------------------
#################################################################
proc RunDelayTest6 {} \
{
   global sender receiver host

   global SGN

   global dly        #" dly: general ptions for all the defined flows
   global flow6      #" flow: specific options for the defined flow6(param,flow_number)
   global mod6
   global customParamsD customEndD
   global runB6 logDelayLB6
   global mID dID  remoteID command comando
   global stopDrecCmd stopMgenCmd stopDrecPid stopMgenPid
   global success
   global getDrecLog
   global decodepath

   global KeySocket
   global socksize
   global KeySource
   global ipv6SourceAddress
   global _sshCMD

   global debug
   global scripted	;# Take away all TK Stuff or not

   if ![info exists scripted] {
      set scripted 0
   }

   if $debug {
      puts stderr "RunDelayTest6: BEGIN"
   }
   
   #check if there is at least one flow6 defined:
   if !$flow6(number) \
   {
     #0 flows
     if !$scripted {
        $logDelayLB6 insert end "Error must define at least one flow."
        update idletasks
     }
     return
   }

   #set well done test flag to 0:
   set success 0

   if !$scripted {
      $runB6 config -text "Wait..."
      $logDelayLB6 delete 0 end
      update idletasks
   }

   #find sender and receiver:
   if $sender(local,ipv6,unicast) {
      ####For the use of the SSH in  IPv6
      set senderHost $host(address,ipv6,unicast)
      set mgenCmd     "$dly(mgenPath,ipv6)"
      set gscriptCmd  "cat > /tmp/gscript"
      set stopMgenCmd "killall $SGN mgen6"
   } else {
	####For the use of the SSH in  IPv6
	set senderHost $sender(address,ipv6,unicast)
	set mgenCmd       "$_sshCMD -l $sender(login,ipv6,unicast) $senderHost $dly(mgenPath,ipv6) "
	set gscriptCmd    "$_sshCMD -l $sender(login,ipv6,unicast) $senderHost {cat>/tmp/gscript}"
	set stopMgenCmd   "$_sshCMD -l $sender(login,ipv6,unicast) $senderHost killall $SGN mgen6"
	  
	set toSend(login) $sender(login,ipv6,unicast)
	set toSend(host) $senderHost
	set toSend(path) $dly(mgenPath,ipv6)
	if !$scripted {
		$logDelayLB6 insert end "Verifying remote mgen..."
		update idletasks
	}
	if { [ verifyRemotePath toSend ] != 0} {
		if !$scripted {
			$logDelayLB6 insert end "Error with remote mgen..."
			$runB6 config -text "Run Test" -command RunDelayTest6
		}
		return
        }
	if !$scripted {
	  	$logDelayLB6 insert end "Remote mgen executable found successfully..."
	  	update idletasks
	}
   }

   if $debug {
      if $sender(local,ipv6,unicast) {
         puts stderr "                    sender local"
      }
      puts stderr "                    senderHost=$senderHost"
      puts stderr "                    mgenCmd=$mgenCmd"
      puts stderr "                    gscriptCmd=$gscriptCmd"
      puts stderr "                    stopMgenCmd=$stopMgenCmd\n"
   }

   ##################################################################
   # NOTE over MGEN-DREC SIGINT: 
   # If you don't use SIGINT drec don't finish to write the Received
   # buffer missing the last 5% of output data!!!!
   # also MGEN don't put statistics information :(
   ##################################################################
   if $receiver(local,ipv6) {
      ####For the use of the SSH in  IPv6
      set receiverHost $host(address,ipv6,unicast)
      set drecCmd $dly(drecPath,ipv6)
      set dscriptCmd  "cat > /tmp/dscript"
      set stopDrecCmd "killall $SGN drec6"
      set getDrecLog  "$decodepath /tmp/output_file.bin | cat"
   } else {
	####For the use of the SSH in  IPv6
	set receiverHost $receiver(address,ipv6,unicast)
	set drecCmd     "$_sshCMD -l $receiver(login,ipv6)  $receiverHost $dly(drecPath,ipv6) "
	set dscriptCmd  "$_sshCMD -l $receiver(login,ipv6)  $receiverHost {cat >/tmp/dscript}"
	set getDrecLog  "$_sshCMD -l $receiver(login,ipv6)  $receiverHost $decodepath /tmp/output_file.bin | cat"
	set stopDrecCmd "$_sshCMD -l $receiver(login,ipv6)  $receiverHost killall $SGN drec6"
	set toSend(login) $receiver(login,ipv6)
	set toSend(host) $receiverHost
	set toSend(path) $dly(drecPath,ipv6)
	if !$scripted {
		$logDelayLB6 insert end "Verifying remote drec..."
		update idletasks
	}
	if { [ verifyRemotePath toSend ] != 0} {
		if !$scripted {
			$logDelayLB6 insert end "Error with remote drec..."
			$runB6 config -text "Run Test" -command RunDelayTest6
		}
		return
	}
	if !$scripted {
		$logDelayLB6 insert end "Remote drec executable found successfully..."
		update idletasks
	}
   }

   if $debug {
      if $receiver(local,ipv6) {
         puts stderr "                    receiver local"
      }
      puts stderr "                    receiverHost=$receiverHost"
      puts stderr "                    drecCmd=$drecCmd"
      puts stderr "                    dscriptCmd=$dscriptCmd"
      puts stderr "                    getDrecLog=$getDrecLog"
      puts stderr "                    stopDrecCmd=$stopDrecCmd\n"
   }
   if !$scripted {
   	$logDelayLB6 insert end "Verifying remote decode..."
	update idletasks
   }
   set mytemp(login) $receiver(login,ipv6)
   set mytemp(host) $receiverHost
   set mytemp(path) $decodepath
   if { [ verifyRemotePath mytemp ] != 0 } {
   	$logDelayLB6 insert end "Remote decode failed"
	$runB6 config -text "Run Test" -command RunDelayTest6
	update idletasks
	return
   }
	if !$scripted {
		$logDelayLB6 insert end "Remote decode found succesfully"
		update idletasks
	}
   
   set receiverHost $host(address,ipv6,unicast)

   #mgen interface
   if $sender(local,ipv6,unicast) {
      set sInterface $host(interface,ipv6,unicast)
   } else {
      set sInterface $sender(interface,ipv6,unicast)
   }

   if {[string trim $sInterface] != ""} {
      append mgenCmd " -i $sInterface "
   }

   append mgenCmd " /tmp/gscript "
   append drecCmd " /tmp/dscript /tmp/output_file.bin"

   if !$scripted {
	$logDelayLB6 insert end "Building scripts..."
	$logDelayLB6 see end
	update idletasks
   }

   lappend gscriptList "# mgen launching script"
   lappend dscriptList "# drec launching script"

   if $debug {
	puts stderr "                    Mgen      command:  $mgenCmd"
	puts stderr "                    GSCRIPT   command:  $gscriptCmd"
	puts stderr "                    Stop MGen command:  $stopMgenCmd"
	puts stderr "                    DSCRIPT   command:  $dscriptCmd"
	puts stderr "                    DREC      command:  $drecCmd"
	puts stderr "                    STOP_DREC command:  $stopDrecCmd"
	puts stderr "                    GETlog    command:  $getDrecLog"
   }

   #if we have a start command
   if $dly(startBool) {
       #[trim]->kill space & tab chars
       #if ($dly(start)!="") then .....
       if {[string trim $dly(start)] != ""} {
          lappend gscriptList "START $dly(start)"
          lappend dscriptList "START $dly(start)"
       }
   }

   #if we have a srcPort command for mgen
   #the port for the source flow6 is ONE
   #for drec we have to set the ports for all the flows
   if $dly(srcPortBool) {
       if {[string trim $dly(srcPort)] != ""} {
          lappend gscriptList "PORT $dly(srcPort)"
       }
   }
   if $KeySocket {
       if {[string trim  $socksize] != ""} {
          lappend gscriptList "SOCKBUFSIZE $socksize"
       }
   }
   if $KeySource {
       if {[string trim $ipv6SourceAddress] != ""} {
          lappend gscriptList "SOURCE $ipv6SourceAddress"
       }
   }

   #drec interface
   if $receiver(local,ipv6) {
      set rInterface $host(interface,ipv6,unicast)
   } else {
      set rInterface $receiver(interface,ipv6)
   }

   if {[string trim $rInterface] != ""} {
      lappend dscriptList "INTERFACE $rInterface"
   }

   #more flow6 with different destination port
   #ports listened by drec:
   for {set i 1} {$i <= $flow6(number)} {incr i} {
      lappend portList $flow6(port,$i)
   }

   #this loop will run at least once,
   #so portList will certainly exist
   lappend dscriptList "PORT [join $portList {,}]"

   ###################################################################
   #tk_messageBox -message "DREC Script List:\n  $dscriptList"
   #tk_messageBox -message "MGEN Script List:\n  $gscriptList"
   ###################################################################

   #deal with flows:
   #construct two temporary lists with commands that
   #will be sorted chronologically afterwards (see the start time)
   lappend drecTempList ""
   for {set i 1} {$i <= $flow6(number)} {incr i} \
   {
      set unresv 0
      #temporary line
      set tempLine "$flow6(start,$i) $i ON $receiverHost  $flow6(port,$i) \
         $flow6(pattern,$i) $flow6(rate,$i) $flow6(size,$i)"
      lappend mgenTempList $tempLine 
	#the flow(i) use TYPE of SERVICE parameters ?
	if $flow6(tosBool,$i) {
		if {[string trim $flow6(tos,$i)] != ""} {
			set TOSLine "$flow6(start,$i) $i ON TCLASS $flow6(tos,$i)"
		}
		lappend mgenTempList $TOSLine
	}

      #note: a generic flux(i) is linked with $mod6(number,$i) modify flows
      #deal with MOD data
      for {set j 1} {$j <= $mod6(number,$i)} {incr j} {
         set tempLine "$mod6(start,$j,$i) $i MOD $receiverHost  $flow6(port,$i) \
           $mod6(pattern,$j,$i) $mod6(rate,$j,$i) $mod6(size,$j,$i)"
         #the mod6(i)  NO use TYPE of SERVICE parameters 
	if $mod6(tosBool,$j,$i) {
		if {[string trim $mod(tos,$j,$i)] != ""} {
			append tempLine " TCLASS $mod(tos,$j,$i)"
		}
	}
         lappend mgenTempList $tempLine
      }
      lappend mgenTempList "$flow6(stop,$i) $i OFF"
   }
   #sort mgenTempList and drecTempList
   if [catch {lsort -index 0 -integer $mgenTempList} mgenTempList] \
   {
	if $debug {
		puts stderr "               ERROR - All start-stop flow values must be integers"
	}
   	if !$scripted {
		$logDelayLB6 insert end "ERROR - All start-stop flow values must be integers"
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
		tk_messageBox -message "All flows start-stop values\nmust be integers"
	}
	return
   }

   #all time values are integers
   #no need to catch next command (uses the same start-stop values)
   if {[string trim $drecTempList] != ""} \
   {
      set drecTempList [lreplace $drecTempList 0 0]
      set drecTempList [lsort -index 0 -integer $drecTempList]
      set dscriptList  [concat $dscriptList $drecTempList]
   }
   set gscriptList [concat $gscriptList $mgenTempList]
   if !$scripted {
	$logDelayLB6 insert end "Done"
	$logDelayLB6 see end
	update idletasks
   }

   ###################################################################
   #tk_messageBox -message "DREC Script List:\n  $dscriptList"
   #tk_messageBox -message "MGEN Script List:\n  $gscriptList"
   ###################################################################
   #send scripts to their right place:
   if !$scripted {
	$logDelayLB6 insert end "Sending mgen6 script..."
	$logDelayLB6 see end
	update idletasks
   }
   ###################################################################
   #tk_messageBox -message "send (mgen script)  command:\n  $gscriptCmd"
   #tk_messageBox -message "send (drec script)  command:\n  $dscriptCmd"
   ###################################################################

   if $debug {
      puts stderr "               Sending mgen script Command: $gscriptCmd"
   }

   if [catch {open "|$gscriptCmd" w} fID] \
   {
	if $debug {
		puts stderr "               ERROR sending mgen Script: $fID"
	}
	if !$scripted {
		tk_messageBox -message $fID
		$runB6 config -text "Run Test" -command RunDelayTest6
	}
	return
   }

   foreach elem $gscriptList {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] {
   	if $debug {
		puts stderr "                  ERROR: $err"
	}
	if !$scripted {
		tk_messageBox -message $err
		$runB6 config -text "Run Test" -command RunDelayTest6
	}
	return
   }

   if !$scripted {
	$logDelayLB6 insert end "Done"
	$logDelayLB6 insert end "Sending drec6 script..."
	$logDelayLB6 see end
	update idletasks
   }

   if $debug {
       puts stderr "               Finished Sending mgen6 script" 
       puts stderr "               Sending drec6 script Command: $dscriptCmd" 
   }

   if [catch {open "|$dscriptCmd" w} fID] \
   {
	if $debug {
		puts stderr "               ERROR: $fID"
	}
	if !$scripted {
		tk_messageBox -message $fID
		$runB6 config -text "Run Test" -command RunDelayTest6
		$logDelayLB6 insert end "ERROR - Sending drec6 script."
		update idletasks
	}
	return
   }
   #else, transfer data from list to script
   #flush $fID

   foreach elem $dscriptList \
   {
      #Write from $elem to a channel($fID)
      puts $fID $elem
   }
   #flush $fID
   if [catch {close $fID} err] \
   {
	if $debug {
		puts stderr "               ERROR: $err"
	}
	if !$scripted {
		tk_messageBox -message $err
		$runB6 config -text "Run Test" -command RunDelayTest6
		$logDelayLB6 insert end "ERROR - Sending drec6 script."
	}
	return
   }
   if !$scripted {
	$logDelayLB6 insert end "Done"
	$logDelayLB6 see end
	update idletasks
   }

   ###################################################################
   #tk_messageBox -message "(mgen) run command:\n  $mgenCmd"
   #tk_messageBox -message "(drec)  run command:\n  $drecCmd"         
   ###################################################################

   #change button:
   if !$scripted {
	$runB6 config -text Stop -command StopMgen6
	$logDelayLB6 insert end "Starting drec6..."
	$logDelayLB6 see end
	update idletasks
   }
   if [catch {open "|$drecCmd" r+ } dID] \
   {
	if $debug {
		puts stderr "               +++++ERROR+++++ Starting drec6" 
		puts stderr "$dID"
	}
	if !$scripted {
		tk_messageBox -message "$dID"
		$runB6 config -text "Run Test" -command RunDelayTest6
		$logDelayLB6 insert end "ERROR - Starting drec6: $dID"
		update idletasks
	}
	return
   }
   if !$scripted {
	$logDelayLB6 insert end "Done"
	$logDelayLB6 see end
	update idletasks
   }
   fconfigure $dID -blocking false
   fileevent $dID readable {
     if {[eof $dID]} {
         #to be sure:
         StopMgen6
	 if !$scripted {
            $logDelayLB6 insert end "Mgen stopped due a drec error"
            $logDelayLB6 see end
            update idletasks
	 }
	catch {close $dID}
	return
      } else {
         gets $dID line
         if {$success!=1} {
	    if !$scripted {
               $logDelayLB6 insert end $line
               $logDelayLB6 see end
               update idletasks
	    }
         }
      }
   }
   if $debug {
      puts stderr "               Starting mgen6 with command: $mgenCmd"
   }

   if !$scripted {
	$logDelayLB6 insert end "Starting mgen6..."
	$logDelayLB6 see end
	update idletasks
   }

   if [catch {open "|$mgenCmd" r+ } mID] \
   {
	if $debug {
		puts stderr "               +++++ERROR+++++ Starting MGEN6, program Output:"
		puts stderr "$mID"
	}
	#error starting mgen
	StopDrec6
	if !$scripted {
		tk_messageBox -message $mID
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
	return
   }

   if !$scripted {
	$logDelayLB6 insert end "Done"
	$logDelayLB6 see end
	update idletasks
   }

   set success "0"
   set finish_mgen 0
   fconfigure $mID -blocking true
   while {![eof $mID]} {
	gets $mID line
	if ![string compare "MGEN: Done." $line] {
		if $debug {
			puts stderr "             Test completed successfully"
		}
		if !$scripted {
			$logDelayLB6 insert end "Test completed successfully"
			$logDelayLB6 see end
			update idletasks
		}
		#save the remote files in management station
		close $dID
		StopDrec6
		SaveDelayData6
		set finish_mgen 1
	}
	if !$scripted {
		$logDelayLB6 insert end $line
		$logDelayLB6 see end
		update idletasks
	}
   }
	if !$finish_mgen {
		catch {close $dID}
		StopDrec6
		$logDelayLB6 insert end stderr "Verify the destination Mgen IP in the Main Options Tab"
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
		if !$scripted {
			tk_messageBox -message "Verify the destination Mgen IP in the Main Options Tab"
		}
	}
		
	if [catch {close $mID} err] {
		if $debug {
			puts stderr "               ERROR: $err"
		}
		if !$scripted {
			tk_messageBox -message $err
			$logDelayLB6 insert end $err
			$logDelayLB6 see end
		}
	}
	if $debug {
			puts stderr "               mgen6 Stopped"
	}
	if !$scripted {
		$logDelayLB6 insert end "MGen6 stopped"
		$logDelayLB6 see end
		update idletasks
	}
   if $debug {
	puts stderr "RunDelayTest6: END"
   }
}

###################################################################
#------------------------------------------------------------------
#                       STOP DREC
#------------------------------------------------------------------
###################################################################
proc StopDrec6 {} {
   global runB6 logDelayLB6 success 
   global stopDrecCmd

   global debug scripted

   ##stop drec:
   if $debug {
       puts stderr "StopDrec6 BEGIN:"
       puts stderr "                Stopping Drec6"
   }
   
   if [catch {open "|$stopDrecCmd" r+ } err] {
	if $debug {
		puts stderr "              ERROR: $err"
	}
	if !$scripted {
		$logDelayLB6 insert end  $err
		$logDelayLB6 see end
		update idletasks
	}
	return
   }
   if $debug {
       puts stderr "              Drec6 succesfully stopped"
       puts stderr "StopDrec6 END:"
   }
}

###################################################################
#------------------------------------------------------------------
#                       STOP MGEN
#------------------------------------------------------------------
###################################################################
proc StopMgen6 {} \
{
   global logDelayLB6 stopMgenCmd
   global runB6
   global debug scripted

   if $debug {
       puts stderr "StopMgen6 BEGIN:"
       puts stderr "              Stopping Mgen"
   }

   if [catch {open "|$stopMgenCmd" r+ } err] {
	if !$scripted {
		tk_messageBox -message $err
		$logDelayLB6 insert end  $err
		$logDelayLB6 see end
		update idletasks
	}
   }
   if $debug {
       puts stderr "              Mgen6 succesfully stoped"
       puts stderr "StopMgen6 END:"
   }
   if !$scripted {
	$runB6 config -text "Run Test" -command RunDelayTest6
   }
}

###################################################################
#------------------------------------------------------------------
#         SAVEDELAYDATA (save result .DLY and other data)
#    save the files from remote station in management station
#------------------------------------------------------------------
###################################################################
proc SaveDelayData6 {} \
{
   global success runB6 logDelayLB6 getDrecLog
   global sID scenarioDir dly fPath

   global debug scripted

   set fPath ""

   if $debug {
	puts stderr "SaveDelayData6: BEGIN"
   }
   if !$scripted {
	$runB6 config -text "Wait..." -command { }
	$logDelayLB6 insert end "Saving data..."
	$logDelayLB6 see end
	update idletasks
   }

   #construct path for local file:
   #the "data" directory:
   set path2 data
   #scenario specific directory:
   set path3 [string trim [string trim $scenarioDir(ipv6,unicast)] {/}]
   #file name:
   set path4 [string trim [string trim $dly(resultsFile,ipv6)] {/}]

   if [string compare {} $path3] {
      set fPath "$path2/$path3/$path4"
   } else {
      #no scenario dir specified:
      set fPath "$path2/$path4"
   }

   #see if user has specified a file name
   #if not, save to default.dly
   if ![string compare {} $path4] \
   {
      lappend fPath default.dly
      #update path
      set dly(resultsFile) default.dly
   }

   #append file extension, if needed
   if [string compare {.dly} [file extension $fPath]] \
   {
      lappend fPath .dly
   }
   #find the parent directory for this file, to create it
   set dPath [file dirname $fPath]

   #try to create this dir
   if [catch {file mkdir $dPath} dErr] \
   {
   	if $debug {
		puts stderr "                     ERROR: Failed creating directory structure"
	}
	if !$scripted {
		#error creating directory (if it already exists, no error)
		$logDelayLB6 insert end "Failed creating directory structure"
		$logDelayLB6 see end
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
	return
   }

   #########################################################################
   #tk_messageBox -message "dPath : $dPath "
   #tk_messageBox -message "fpath : $fPath "
   #tk_messageBox -message "getDrecLog : $getDrecLog "
   #########################################################################

   #check if the file already exists:
   if [file exists $fPath] \
   {
	if !$scripted {
		set answer [tk_messageBox -message "File already exists: Overwrite?" \
		   -type yesno -icon question]
	} else {
		#script always overwrite
		set answer yes
	}
	if [string compare $answer yes] {
		if !$scripted {
			#user says no:
			$logDelayLB6 delete 0 end
			$logDelayLB6 insert end "Change results file name"
			$logDelayLB6 insert end "and press Save Data button"
			$logDelayLB6 see end
			$runB6 config -text "Save Data" -command SaveDelayData6
			update idletasks
		}
		return
	   }
	   #else, it's ok to continue
   }
   #try to open file in write mode:
   if [catch {open $fPath w} iD] \
   {
   	if $debug {
		puts stderr "                     ERROR: Failed opening local file"
	}
   	if !$scripted {
		#error opening file
		$logDelayLB6 insert end "Failed opening local file"
		$logDelayLB6 see end
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
	return
   }
   #else, we can write.
   #open pipe to read file:

   if [catch {open "|$getDrecLog" r+ } sID] \
   {
	if $debug {
		puts stderr "                     ERROR: Failed opening remote data file"
	}
	if !$scripted {
		#error opening file
		$logDelayLB6 insert end "Failed opening remote data file"
		$logDelayLB6 see end
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
	close $iD
	return
   }
   #else:
   #The fcopy command copies data from one I/O channel,
   #inchan to another I/O channel  in background;
   #if an error occur command is executed
   #sID=socket ID
   #fcopy return nbytes and in case error
   #this is the default implementation of the fcopy (see help)

   fconfigure $sID -translation binary
   fconfigure $iD -translation binary
#   fcopy $sID $iD -command [list Finished6 $sID $iD]
   set bytes [fcopy $sID $iD]
   Finished6 $sID $iD $bytes

   if $debug {
	puts stderr "SaveDelayData6: END"
   }
}

###################################################################
#------------------------------------------------------------------
#         when we finish the remote copy to local file
#------------------------------------------------------------------
###################################################################
proc Finished6 {in out bytes {error {}} } \
{
   global logDelayLB6 runB6
   global debug scripted

   if $debug {
      puts stderr "Finished6: BEGIN"
   }

   if !$scripted {
	$logDelayLB6 insert end "$bytes bytes transferred"
	$logDelayLB6 see end
   }

	if {$bytes == 0} {
		if $debug {
			puts stderr "                 -------ERROR--------: Test Failed empty data file found!"
		}
		if !$scripted {
			tk_messageBox -message "Test Failed empty data file found\nPlease check Source, Destination or Directory Permissions"
			$runB6 config -text "Run Test" -command RunDelayTest6
			update idletasks
		}
   		close $in
   		close $out
		return
	}

   if [string length $error] {
	if $debug {
		puts stderr "                ERROR: $error"
	}
	if !$scripted {
		$logDelayLB6 insert end $error
		$logDelayLB6 see end
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
   } else {
	if !$scripted {
		#at last:
		$logDelayLB6 insert end "Done"
		$logDelayLB6 see end
	}
	ProcessData6
   }

   if $debug {
      puts stderr "Finished6: END"
   }

   close $in
   close $out
}

###################################################################
#------------------------------------------------------------------
#   process the local data file (.DLY) 
#------------------------------------------------------------------
###################################################################
proc ProcessData6 {} \
{
   global runB6 logDelayLB6 fPath mcID
   global mcalcpath
 
   global debug scripted
   
   catch {unset outList}
   
   if $debug {
	puts stderr "ProcessData6: BEGIN"
   }

   if !$scripted {
	#announce the beginning
	$logDelayLB6 insert end "Processing data..."
	$logDelayLB6 see end
	update idletasks
   }

   set McalcCmd "$mcalcpath <$fPath"

   if [catch { open "|$McalcCmd"} mcID] \
   {
	if !$scripted {
		$logDelayLB6 insert end "Processing failed"
		$logDelayLB6 see end
		$runB6 config -text "Run Test" -command RunDelayTest6
		update idletasks
	}
	return
   }
   set outList ""
   fconfigure $mcID -blocking true
   #else, read output in memory:
   while {![eof $mcID]} {
         gets $mcID line
         lappend outList $line
   }
   #when done:
   catch {close $mcID}
   SaveMcalcData6 $outList
   unset outList
   if $debug {
	puts stderr "ProcessData6: END"
   }
}

###################################################################
#------------------------------------------------------------------
#                       SAVE MCALC DATA  (.MCL)
#------------------------------------------------------------------
###################################################################
proc SaveMcalcData6 { outList } {
	global runB6 logDelayLB6 flow6 dly mod6
	global fPath
	global xID
	global paramU6 pU6 countPU6

	global debug scripted

	if $debug {
		puts stderr "SaveMcalcData6: BEGIN"
	}
	#update flow6 data
	for {set i 1} {$i <= $flow6(number)} {incr i} {
		set flow6(active,$i) no
		set flow6(mod,$i) $mod6(number,$i)
	}

	set ok 0
	set lineCount 0

	#show mcalc results
	if !$scripted {
		foreach line_mine $outList {
			$logDelayLB6 insert end $line_mine
		}
		update idletask
	}

	foreach line $outList {
		if {[scan $line "%s" isFlow] != -1} {
			if ![string compare  [string range $line 0 3]  "FLOW"] {
				#a line announcing a flow
				incr ok
				#retrieve flow data
				set count $lineCount

				#flow number, source and destination:
				scan [lindex $outList $count] "FLOW>%d   SOURCE>%s  DESTINATION>%s"   fNr src dst

				set mcalc($fNr,source)       $src
				set mcalc($fNr,destination)  $dst

				#num of packets received
				incr count

				scan [lindex $outList $count] "   Num pkts recvd  : %s" var
				set mcalc($fNr,recvPacketsNumber) $var

				#joined delay:
				incr count
				scan [lindex $outList $count] "   Join delay      : %s" var
				set mcalc($fNr,joinDelay) $var

				#received packet rate:
				incr count
				scan [lindex $outList $count] "   Recv pkt rate   : %s" var
				set mcalc($fNr,recvPacketsRate) $var

				#recv data rate
				incr count
				scan [lindex $outList $count] "   Recv data rate  : %s" var
				set mcalc($fNr,recvDataRate) $var

				#packets dropped:
				incr count
				scan [lindex $outList $count] "   Pkts dropped    : %s" var
				set mcalc($fNr,packetsDropped) $var

				#average delay
				incr count
				scan [lindex $outList $count] "   Ave. Tx Delay   : %s" var
				set mcalc($fNr,averageDelay) $var

				#max delay
				incr count
				scan [lindex $outList $count] "   Max. Tx Delay   : %s" var
				set mcalc($fNr,maxDelay) $var

				#min delay
				incr count
				scan [lindex $outList $count] "   Min. Tx Delay   : %s" var
				set mcalc($fNr,minDelay) $var

				#delay variation:
				incr count
				scan [lindex $outList $count] "   Delay variation : %s" var
				set mcalc($fNr,delayVariation) $var

				#update flow array:
				set flow6(active,$fNr) yes
			};#end if FLOW:
			incr lineCount
		}
	};#end foreach

	#####################################################################
	# Finally Save the DATA file
	# set file name: the same as fPath, but different extension:.mcl file
	#####################################################################
	set mPath [file rootname $fPath]
	append mPath .mcl

	#open file
	if [catch {open $mPath w} xID] {
		if $debug {
			puts stderr "                     ERROR - Saving results: $xID"
		}
		if !$scripted {
			$logDelayLB6 insert end "Failed saving results"
			$logDelayLB6 see end
			$runB6 config -text "Run Test" -command RunDelayTest6
			update idletasks
		}
		return
	}
   #else, start writing data:
   for {set i 1} {$i <= $flow6(number)} {incr i} \
   {
      puts $xID " "
      puts $xID "Mgen measure"
      puts $xID "Flow name: $dly(title,ipv6) $flow6(name,$i)"
      puts $xID " "
      puts $xID "Flow IPv6 ID $i"
      puts $xID "-Flow Relative Start (msec):  $flow6(start,$i)"
      puts $xID "-Flow Relative Stop  (msec):   $flow6(stop,$i)"
      puts $xID "Pattern  $flow6(pattern,$i)"
      puts $xID "-Send Rate (packets/sec): $flow6(rate,$i)"
      puts $xID "-UDP Packet Size (bytes): $flow6(size,$i)"
      puts $xID "-IP Packet Size (bytes): [expr $flow6(size,$i) +48]"
      puts $xID "Nr. of MODs   $flow6(mod,$i)"
	if $flow6(tosBool,$i) {
		puts $xID "IP TOS Value  $flow6(tos,$i)"
	}
      
      if $dly(startBool) {
         puts $xID "Mgen Absolute START   $dly(start)"
      }
      #####################################################
      # MCALC data
      #####################################################
      puts $xID "Flow Source        $mcalc($i,source)"
      puts $xID "Flow Destination   $mcalc($i,destination)"
      puts $xID "-Received packets: $mcalc($i,recvPacketsNumber)"
      puts $xID "-Join delay (sec): $mcalc($i,joinDelay)"
      puts $xID "-Recv rate (packets/sec): $mcalc($i,recvPacketsRate)"
      puts $xID "-Recv data rate (kbps): $mcalc($i,recvDataRate)"
      puts $xID "-Packets dropped: $mcalc($i,packetsDropped)"
      puts $xID "-Average delay(sec):   $mcalc($i,averageDelay)"
      puts $xID "-Max delay (sec): $mcalc($i,maxDelay)"
      puts $xID "-Min delay (sec): $mcalc($i,minDelay)"
      puts $xID "-Delay variation (sec): $mcalc($i,delayVariation)"
      #custom params:
      set first 1
      set selList [lsort [array names paramU6]]
      for {set count 0} {$count < $countPU6} {incr count} \
      {
         set tmp [lindex $selList $count]
         if [string compare "(parameters)" $tmp] \
         { 
            if { $first == 1 } { puts $xID "Custom parameters:" }
            puts $xID "-$tmp" 
            set first 0 
         }
      }

   }
   close $xID
   if !$scripted {
	$logDelayLB6 insert end "Done "
	$logDelayLB6 see end
	$runB6 config -text "Run Test" -command RunDelayTest6
	update idletasks
   }
   if $debug {
 	puts stderr "SaveMcalcData6: END"
   }
}

